#include <torch/extension.h>

#include <vector>

// CUDA forward declarations

std::vector<torch::Tensor> cm_feeder_cuda_forward(
    const int n_modules,
    torch::Tensor input);

std::vector<torch::Tensor> cm_feeder_cuda_backward(
    const int n_modules,
  torch::Tensor output_grad);

// C++ interface

#define CHECK_CUDA(x) TORCH_CHECK(x.device().is_cuda(), #x " must be a CUDA tensor")
#define CHECK_CONTIGUOUS(x) TORCH_CHECK(x.is_contiguous(), #x " must be contiguous")
#define CHECK_INPUT(x) CHECK_CUDA(x); CHECK_CONTIGUOUS(x)

std::vector<torch::Tensor> cm_feeder_forward(
    const int n_modules,
    torch::Tensor input)
{
  CHECK_INPUT(input);
  return cm_feeder_cuda_forward(n_modules, input);
}

std::vector<torch::Tensor> cm_feeder_backward(
  const int n_modules,
  torch::Tensor output_grad)
  {

  CHECK_INPUT(output_grad);

  return cm_feeder_cuda_backward(
    n_modules,
    output_grad);
}

PYBIND11_MODULE(TORCH_EXTENSION_NAME, m)
 {
  m.def("forward", &cm_feeder_forward, "CM Feeder forward (CUDA)");
  m.def("backward", &cm_feeder_backward, "CM Feeder backward (CUDA)");
}